@@ -3,7 +3,7 @@
  *   LinuxSampler - modular, streaming capable sampler                     *
  *                                                                         *
  *   Copyright (C) 2003, 2004 by Benno Senoner and Christian Schoenebeck   *
- *   Copyright (C) 2005 - 2013 Christian Schoenebeck                       *
+ *   Copyright (C) 2005 - 2025 Christian Schoenebeck                       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -70,7 +70,7 @@ namespace LinuxSampler {
                     virtual bool   Mandatory() OVERRIDE;
                     virtual std::map<String,DeviceCreationParameter*> DependsAsParameters() OVERRIDE;
                     virtual optional<bool> DefaultAsBool(std::map<String,String> Parameters) OVERRIDE;
-                    virtual void OnSetValue(bool b) throw (Exception) OVERRIDE;
+                    virtual void OnSetValue(bool b) LS_THROWS(Exception) OVERRIDE;
                     static String Name();
             };
 
@@ -91,7 +91,7 @@ namespace LinuxSampler {
                     virtual optional<int>    RangeMaxAsInt(std::map<String,String> Parameters) OVERRIDE;
                     virtual std::vector<int> PossibilitiesAsInt(std::map<String,String> Parameters) OVERRIDE;
                     virtual int              ValueAsInt() OVERRIDE;
-                    virtual void             OnSetValue(int i) throw (Exception) OVERRIDE;
+                    virtual void             OnSetValue(int i) LS_THROWS(Exception) OVERRIDE;
                     static String Name();
             };
 
@@ -112,7 +112,7 @@ namespace LinuxSampler {
                     virtual optional<int>    RangeMinAsInt(std::map<String,String> Parameters) OVERRIDE;
                     virtual optional<int>    RangeMaxAsInt(std::map<String,String> Parameters) OVERRIDE;
                     virtual std::vector<int> PossibilitiesAsInt(std::map<String,String> Parameters) OVERRIDE;
-                    virtual void             OnSetValue(int i) throw (Exception) OVERRIDE;
+                    virtual void             OnSetValue(int i) LS_THROWS(Exception) OVERRIDE;
                     static String Name();
             };
 
@@ -286,7 +286,7 @@ namespace LinuxSampler {
              *
              * @throws Exception - if given send effect chain doesn't exist
              */
-            void RemoveSendEffectChain(uint iChain) throw (Exception);
+            void RemoveSendEffectChain(uint iChain) LS_THROWS(Exception);
 
             /**
              * Returns send effect chain given by @a iChain or @c NULL if
@@ -314,7 +314,7 @@ namespace LinuxSampler {
             /**
              * @deprecated This method will be removed, use RemoveSendEffectChain() instead!
              */
-            void RemoveMasterEffectChain(uint iChain) throw (Exception) DEPRECATED_API;
+            void RemoveMasterEffectChain(uint iChain) LS_THROWS(Exception) DEPRECATED_API;
 
             /**
              * @deprecated This method will be removed, use SendEffectChain() instead!
