@@ -3,7 +3,7 @@
  *   LinuxSampler - modular, streaming capable sampler                     *
  *                                                                         *
  *   Copyright (C) 2009 Grigor Iliev                                       *
- *   Copyright (C) 2013-2016 Christian Schoenebeck                         *
+ *   Copyright (C) 2013-2025 Christian Schoenebeck                         *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -83,14 +83,14 @@ namespace LinuxSampler {
             class ParameterDevice : public DeviceCreationParameterString {
                 public:
                     ParameterDevice();
-                    ParameterDevice(String s) throw (Exception);
+                    ParameterDevice(String s) LS_THROWS(Exception);
                     virtual String Description() OVERRIDE;
                     virtual bool   Fix() OVERRIDE;
                     virtual bool   Mandatory() OVERRIDE;
                     virtual std::map<String,DeviceCreationParameter*> DependsAsParameters() OVERRIDE;
                     virtual optional<String>    DefaultAsString(std::map<String,String> Parameters) OVERRIDE;
                     virtual std::vector<String> PossibilitiesAsString(std::map<String,String> Parameters) OVERRIDE;
-                    virtual void                OnSetValue(String s) throw (Exception) OVERRIDE;
+                    virtual void                OnSetValue(String s) LS_THROWS(Exception) OVERRIDE;
                     int GetDeviceIndex();
                     static String Name();
 
@@ -131,7 +131,7 @@ namespace LinuxSampler {
             class ParameterBuffers : public DeviceCreationParameterInt {
                 public:
                     ParameterBuffers();
-                    ParameterBuffers(String s) throw (Exception);
+                    ParameterBuffers(String s) LS_THROWS(Exception);
                     virtual String Description() OVERRIDE;
                     virtual bool   Fix() OVERRIDE;
                     virtual bool   Mandatory() OVERRIDE;
@@ -140,7 +140,7 @@ namespace LinuxSampler {
                     virtual optional<int>    RangeMinAsInt(std::map<String,String> Parameters) OVERRIDE;
                     virtual optional<int>    RangeMaxAsInt(std::map<String,String> Parameters) OVERRIDE;
                     virtual std::vector<int> PossibilitiesAsInt(std::map<String,String> Parameters) OVERRIDE;
-                    virtual void             OnSetValue(int i) throw (Exception) OVERRIDE;
+                    virtual void             OnSetValue(int i) LS_THROWS(Exception) OVERRIDE;
                     static String Name();
             };
 
@@ -151,7 +151,7 @@ namespace LinuxSampler {
             class ParameterBufferSize : public DeviceCreationParameterInt {
                 public:
                     ParameterBufferSize();
-                    ParameterBufferSize(String s) throw (Exception);
+                    ParameterBufferSize(String s) LS_THROWS(Exception);
                     virtual String Description() OVERRIDE;
                     virtual bool   Fix() OVERRIDE;
                     virtual bool   Mandatory() OVERRIDE;
@@ -160,7 +160,7 @@ namespace LinuxSampler {
                     virtual optional<int>    RangeMinAsInt(std::map<String,String> Parameters) OVERRIDE;
                     virtual optional<int>    RangeMaxAsInt(std::map<String,String> Parameters) OVERRIDE;
                     virtual std::vector<int> PossibilitiesAsInt(std::map<String,String> Parameters) OVERRIDE;
-                    virtual void             OnSetValue(int i) throw (Exception) OVERRIDE;
+                    virtual void             OnSetValue(int i) LS_THROWS(Exception) OVERRIDE;
                     static String Name();
             };
 
@@ -178,7 +178,7 @@ namespace LinuxSampler {
             atomic_t            restartQueue;
             Mutex               destroyMutex;
 
-            void CreateAndStartAudioQueue() throw(Exception);
+            void CreateAndStartAudioQueue() LS_THROWS(Exception);
             void DestroyAudioQueue();
             void FillBuffers();
             void PrimeAudioQueue();
