@@ -1,6 +1,7 @@
 /***************************************************************************
  *                                                                         *
  *   Copyright (C) 2008 - 2013 Andreas Persson                             *
+ *   Copyright (C) 2009 - 2025 Christian Schoenebeck                       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -66,7 +67,7 @@ namespace LinuxSampler {
         class ParameterFragmentSize : public DeviceCreationParameterInt {
         public:
             ParameterFragmentSize();
-            ParameterFragmentSize(String s) throw (Exception);
+            ParameterFragmentSize(String s) LS_THROWS(Exception);
             String Description() OVERRIDE;
             bool Fix() OVERRIDE;
             bool Mandatory() OVERRIDE;
@@ -75,7 +76,7 @@ namespace LinuxSampler {
             optional<int> RangeMinAsInt(std::map<String,String> Parameters) OVERRIDE;
             optional<int> RangeMaxAsInt(std::map<String,String> Parameters) OVERRIDE;
             std::vector<int> PossibilitiesAsInt(std::map<String,String> Parameters) OVERRIDE;
-            void OnSetValue(int i) throw (Exception) OVERRIDE;
+            void OnSetValue(int i) LS_THROWS(Exception) OVERRIDE;
             static String Name();
         };
 
