@@ -3,7 +3,7 @@
  *   LinuxSampler - modular, streaming capable sampler                     *
  *                                                                         *
  *   Copyright (C) 2003, 2004 by Benno Senoner and Christian Schoenebeck   *
- *   Copyright (C) 2005 - 2012 Christian Schoenebeck                       *
+ *   Copyright (C) 2005 - 2025 Christian Schoenebeck                       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -82,7 +82,7 @@ void CALLBACK MidiInputDeviceMme::MidiInputPortMme::win32_midiin_callback(HMIDII
         return std::vector<int>();
     }
 
-   void MidiInputDeviceMme::ParameterPorts::OnSetValue(int i) throw (Exception) {
+   void MidiInputDeviceMme::ParameterPorts::OnSetValue(int i) LS_THROWS(Exception) {
         if (i != 1) throw Exception("MME only supports one MIDI port per device");
     }
 
@@ -136,7 +136,7 @@ void CALLBACK MidiInputDeviceMme::MidiInputPortMme::win32_midiin_callback(HMIDII
 // *************** MidiInputPortMme ***************
 // *
 
-    MidiInputDeviceMme::MidiInputPortMme::MidiInputPortMme(MidiInputDeviceMme* pDevice) throw (MidiInputException)
+    MidiInputDeviceMme::MidiInputPortMme::MidiInputPortMme(MidiInputDeviceMme* pDevice) LS_THROWS(MidiInputException)
         : MidiInputPort(pDevice, ((DeviceCreationParameterInt*)pDevice->Parameters["PORTS"])->ValueAsInt() - 1)
     {
         this->pDevice = pDevice;
@@ -339,7 +339,7 @@ void MidiInputDeviceMme::MidiInputPortMme::MmeCallbackDispatcher(HMIDIIN handle,
     }
 
     String MidiInputDeviceMme::Version() {
-        String s = "$Revision: 2494 $";
+        String s = "$Revision: 4167 $";
         return s.substr(11, s.size() - 13); // cut dollar signs, spaces and CVS macro keyword
     }
 
