@@ -1,6 +1,6 @@
 /***************************************************************************
  *                                                                         *
- *   Copyright (C) 2008 - 2013 Christian Schoenebeck                       *
+ *   Copyright (C) 2008 - 2025 Christian Schoenebeck                       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -57,14 +57,14 @@ public:
      *
      * @throw Exception - if given position is invalid
      */
-    void InsertEffect(Effect* pEffect, int iChainPos) throw (Exception);
+    void InsertEffect(Effect* pEffect, int iChainPos) LS_THROWS(Exception);
 
     /**
      * Remove effect at chain position @a iChainPos from the effect chain.
      *
      * @throws Exception - if given position is invalid
      */
-    void RemoveEffect(int iChainPos) throw (Exception);
+    void RemoveEffect(int iChainPos) LS_THROWS(Exception);
 
     /**
      * Sequentially render the whole effects chain. The final signal
@@ -91,7 +91,7 @@ public:
      *
      * @throw Exception - if chain position is invalid
      */
-    void SetEffectActive(int iChainPos, bool bOn) throw (Exception);
+    void SetEffectActive(int iChainPos, bool bOn) LS_THROWS(Exception);
 
     /**
      * Whether the given effect is currently enabled.
