@@ -3,7 +3,7 @@
  *   LinuxSampler - modular, streaming capable sampler                     *
  *                                                                         *
  *   Copyright (C) 2003, 2004 by Benno Senoner and Christian Schoenebeck   *
- *   Copyright (C) 2005, 2007 Christian Schoenebeck                        *
+ *   Copyright (C) 2005 - 2015 Christian Schoenebeck                       *
  *                                                                         *
  *   This library is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -37,7 +37,7 @@ namespace LinuxSampler { namespace gig {
         StepsLeft = (int) (DecayTime * SampleRate);
         Coeff     = (StepsLeft) ? (1.0f - Depth) / (float) StepsLeft : 0.0f;
 
-        dmsg(4,("Depth=%d, DecayTime=%f\n", Depth, DecayTime));
+        dmsg(4,("Depth=%f, DecayTime=%f\n", Depth, DecayTime));
     }
 
 }} // namespace LinuxSampler::gig
