@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2014 - 2020 Christian Schoenebeck
+ * Copyright (c) 2014 - 2025 Christian Schoenebeck
  *
  * http://www.linuxsampler.org
  *
@@ -67,16 +67,39 @@ namespace LinuxSampler {
     }
     #endif
 
-    static vmint _requiredMaxStackSizeFor(Statement* statement, vmint depth = 0) {
-        if (!statement) return 1;
+    struct StackSizeReq {
+        vmint opStackSize;
+        vmint dataStackSize;
+
+        void maxUp(const StackSizeReq& o) {
+            if (o.opStackSize > opStackSize)
+                opStackSize = o.opStackSize;
+            if (o.dataStackSize > dataStackSize)
+                dataStackSize = o.dataStackSize;
+        }
+    };
+
+    static StackSizeReq _requiredMaxStackSizeFor(Statement* statement, vmint depth = 0) {
+        if (!statement) return {
+            .opStackSize = 1,
+            .dataStackSize = 0
+        };
 
         switch (statement->statementType()) {
-            case STMT_LEAF:
+            case STMT_LEAF: {
                 #if DEBUG_SCRIPTVM_CORE
                 _printIndents(depth);
                 printf("-> STMT_LEAF\n");
                 #endif
-                return 1;
+                LeafStatement* leafStmt = (LeafStatement*) statement;
+                StackSizeReq max = {};
+                for (int i = 0; Statement* depStmt = leafStmt->dependencyStatement(i); ++i) {
+                    StackSizeReq size = _requiredMaxStackSizeFor(depStmt, depth+1);
+                    max.maxUp(size);
+                }
+                max.opStackSize++;
+                return max;
+            }
 
             case STMT_LIST: {
                 #if DEBUG_SCRIPTVM_CORE
@@ -84,12 +107,19 @@ namespace LinuxSampler {
                 printf("-> STMT_LIST\n");
                 #endif
                 Statements* stmts = (Statements*) statement;
-                vmint max = 0;
-                for (int i = 0; stmts->statement(i); ++i) {
-                    vmint size = _requiredMaxStackSizeFor( stmts->statement(i), depth+1 );
-                    if (max < size) max = size;
+                StackSizeReq max = {};
+                for (int i = 0; Statement* depStmt = stmts->dependencyStatement(i); ++i) {
+                    StackSizeReq size = _requiredMaxStackSizeFor(depStmt, depth+1);
+                    max.maxUp(size);
                 }
-                return max + 1;
+                for (int i = 0; Statement* nextStmt = stmts->statement(i); ++i) {
+                    StackSizeReq size = _requiredMaxStackSizeFor(nextStmt, depth+1);
+                    max.maxUp(size);
+                }
+                max.opStackSize++;
+                if (stmts->needsNewDataStackFrame())
+                    max.dataStackSize += stmts->dataStackFrameSize();
+                return max;
             }
 
             case STMT_BRANCH: {
@@ -98,12 +128,17 @@ namespace LinuxSampler {
                 printf("-> STMT_BRANCH\n");
                 #endif
                 BranchStatement* branchStmt = (BranchStatement*) statement;
-                vmint max = 0;
-                for (int i = 0; branchStmt->branch(i); ++i) {
-                    vmint size = _requiredMaxStackSizeFor( branchStmt->branch(i), depth+1 );
-                    if (max < size) max = size;
+                StackSizeReq max = {};
+                for (int i = 0; Statement* depStmt = branchStmt->dependencyStatement(i); ++i) {
+                    StackSizeReq size = _requiredMaxStackSizeFor(depStmt, depth+1);
+                    max.maxUp(size);
                 }
-                return max + 1;
+                for (int i = 0; Statements* branchStmts = branchStmt->branch(i); ++i) {
+                    StackSizeReq size = _requiredMaxStackSizeFor(branchStmts, depth+1);
+                    max.maxUp(size);
+                }
+                max.opStackSize++;
+                return max;
             }
 
             case STMT_LOOP: {
@@ -112,10 +147,17 @@ namespace LinuxSampler {
                 printf("-> STMT_LOOP\n");
                 #endif
                 While* whileStmt = (While*) statement;
-                if (whileStmt->statements())
-                    return _requiredMaxStackSizeFor( whileStmt->statements() ) + 1;
-                else
-                    return 1;
+                StackSizeReq max = {};
+                for (int i = 0; Statement* depStmt = whileStmt->dependencyStatement(i); ++i) {
+                    StackSizeReq size = _requiredMaxStackSizeFor(depStmt, depth+1);
+                    max.maxUp(size);
+                }
+                if (Statements* whileStmts = whileStmt->statements()) {
+                    StackSizeReq size = _requiredMaxStackSizeFor(whileStmts);
+                    max.maxUp(size);
+                }
+                max.opStackSize++;
+                return max;
             }
 
             case STMT_SYNC: {
@@ -124,31 +166,51 @@ namespace LinuxSampler {
                 printf("-> STMT_SYNC\n");
                 #endif
                 SyncBlock* syncStmt = (SyncBlock*) statement;
-                if (syncStmt->statements())
-                    return _requiredMaxStackSizeFor( syncStmt->statements() ) + 1;
-                else
-                    return 1;
+                if (Statements* syncStmts = syncStmt->statements()) {
+                    StackSizeReq size = _requiredMaxStackSizeFor(syncStmts);
+                    size.opStackSize++;
+                    return size;
+                } else {
+                    return {
+                        .opStackSize = 1,
+                        .dataStackSize = 0
+                    };
+                }
             }
 
             case STMT_NOOP:
-                break; // no operation like the name suggests
+                #if DEBUG_SCRIPTVM_CORE
+                _printIndents(depth);
+                printf("-> STMT_NOOP\n");
+                #endif
+                return {
+                    .opStackSize = 1,
+                    .dataStackSize = 0
+                };
         }
 
-        return 1; // actually just to avoid compiler warning
+        // actually just to avoid compiler warning
+        return {
+            .opStackSize = 1,
+            .dataStackSize = 0
+        };
     }
 
-    static vmint _requiredMaxStackSizeFor(EventHandlers* handlers) {
-        vmint max = 1;
+    static StackSizeReq _requiredMaxStackSizeFor(EventHandlers* handlers) {
+        StackSizeReq max = {
+            .opStackSize = 1,
+            .dataStackSize = 0
+        };
         for (int i = 0; i < handlers->size(); ++i) {
-            vmint size = _requiredMaxStackSizeFor(handlers->eventHandler(i));
-            if (max < size) max = size;
+            StackSizeReq size = _requiredMaxStackSizeFor(handlers->eventHandler(i));
+            max.maxUp(size);
         }
         return max;
     }
 
     ScriptVM::ScriptVM() :
         m_eventHandler(NULL), m_parserContext(NULL), m_autoSuspend(true),
-        m_acceptExitRes(false)
+        m_acceptExitRes(false), m_deployment(VM_PRODUCTION)
     {
         m_fnMessage = new CoreVMFunction_message;
         m_fnExit = new CoreVMFunction_exit(this);
@@ -358,14 +420,18 @@ namespace LinuxSampler {
     VMExecContext* ScriptVM::createExecContext(VMParserContext* parserContext) {
         ParserContext* parserCtx = dynamic_cast<ParserContext*>(parserContext);
         ExecContext* execCtx = new ExecContext();
-        
+        StackSizeReq requiredStackSize = {};
         if (parserCtx->requiredMaxStackSize < 0) {
-             parserCtx->requiredMaxStackSize =
-                _requiredMaxStackSizeFor(&*parserCtx->handlers);
+            requiredStackSize = _requiredMaxStackSizeFor(&*parserCtx->handlers);
+            parserCtx->requiredMaxStackSize = requiredStackSize.opStackSize;
+            parserCtx->requiredMaxDataStackSize = requiredStackSize.dataStackSize;
         }
-        execCtx->stack.resize(parserCtx->requiredMaxStackSize);
-        dmsg(2,("Created VM exec context with %" PRId64 " bytes VM stack size.\n",
-                int64_t(parserCtx->requiredMaxStackSize * sizeof(ExecContext::StackFrame))));
+        execCtx->opStack.resize(parserCtx->requiredMaxStackSize);
+        execCtx->dataStack.resize(parserCtx->requiredMaxDataStackSize);
+        dmsg(2,("Created VM exec context with %" PRId64 " bytes op stack.\n",
+                int64_t(parserCtx->requiredMaxStackSize * sizeof(ExecContext::OpStackFrame))));
+        dmsg(2,("Allocated %" PRId64 " bytes data stack.\n",
+                int64_t(parserCtx->requiredMaxDataStackSize)));
         //printf("execCtx=0x%lx\n", (uint64_t)execCtx);
         const vmint polyIntSize = parserCtx->polyphonicIntVarCount;
         execCtx->polyphonicIntMemory.resize(polyIntSize);
@@ -512,6 +578,14 @@ namespace LinuxSampler {
         return m_parserContext->execContext;
     }
 
+    void ScriptVM::setDeploymentCase(VMDeploymentCase deployment) {
+        m_deployment = deployment;
+    }
+
+    VMDeploymentCase ScriptVM::getDeploymentCase() const {
+        return m_deployment;
+    }
+
     void ScriptVM::setAutoSuspendEnabled(bool b) {
         m_autoSuspend = b;
     }
@@ -528,7 +602,7 @@ namespace LinuxSampler {
         return m_acceptExitRes;
     }
 
-    VMExecStatus_t ScriptVM::exec(VMParserContext* parserContext, VMExecContext* execContex, VMEventHandler* handler) {
+    VMExecStatus_t ScriptVM::exec(VMParserContext* parserContext, VMExecContext* execContext, VMEventHandler* handler) {
         m_parserContext = dynamic_cast<ParserContext*>(parserContext);
         if (!m_parserContext) {
             std::cerr << "No VM parser context provided. Did you load a script?\n";
@@ -538,7 +612,7 @@ namespace LinuxSampler {
         // a ParserContext object is always tied to exactly one ScriptVM object
         assert(m_parserContext->functionProvider == this);
 
-        ExecContext* ctx = dynamic_cast<ExecContext*>(execContex);
+        ExecContext* ctx = dynamic_cast<ExecContext*>(execContext);
         if (!ctx) {
             std::cerr << "Invalid VM exec context.\n";
             return VMExecStatus_t(VM_EXEC_NOT_RUNNING | VM_EXEC_ERROR);
@@ -549,6 +623,7 @@ namespace LinuxSampler {
 
         m_parserContext->execContext = ctx;
 
+        ctx->deployment = m_deployment;
         ctx->status = VM_EXEC_RUNNING;
         ctx->instructionsCount = 0;
         ctx->clearExitRes();
@@ -556,20 +631,21 @@ namespace LinuxSampler {
         vmint instructionsCounter = 0;
         vmint synced = m_autoSuspend ? 0 : 1;
 
-        int& frameIdx = ctx->stackFrame;
+        int& frameIdx = ctx->opStackFrame;
         if (frameIdx < 0) { // start condition ...
             frameIdx = -1;
-            ctx->pushStack(h);
+            ctx->dataStackPtr = -1;
+            ctx->pushOpStack(h);
         }
 
         while (flags == STMT_SUCCESS && frameIdx >= 0) {
-            if (frameIdx >= ctx->stack.size()) { // should never happen, otherwise it's a bug ...
-                std::cerr << "CRITICAL: VM stack overflow! (" << frameIdx << ")\n";
+            if (frameIdx >= ctx->opStack.size()) { // should never happen, otherwise it's a bug ...
+                std::cerr << "CRITICAL: VM op stack overflow! (" << frameIdx << ")\n";
                 flags = StmtFlags_t(STMT_ABORT_SIGNALLED | STMT_ERROR_OCCURRED);
                 break;
             }
 
-            ExecContext::StackFrame& frame = ctx->stack[frameIdx];
+            ExecContext::OpStackFrame& frame = ctx->opStack[frameIdx];
             switch (frame.statement->statementType()) {
                 case STMT_LEAF: {
                     #if DEBUG_SCRIPTVM_CORE
@@ -577,8 +653,21 @@ namespace LinuxSampler {
                     printf("-> STMT_LEAF\n");
                     #endif
                     LeafStatement* leaf = (LeafStatement*) frame.statement;
-                    flags = leaf->exec();
-                    ctx->popStack();
+                    if (!frame.state) {
+                        Statement* dependencyStmt =
+                            leaf->dependencyStatement(frame.subindex);
+                        if (!dependencyStmt) {
+                            frame.subindex = 0;
+                            frame.state = 1;
+                            goto leaf_state_1;
+                        }
+                        ctx->pushOpStack(dependencyStmt);
+                        ++frame.subindex;
+                    } else {
+                        leaf_state_1:
+                        flags = leaf->exec();
+                        ctx->popOpStack();
+                    }
                     break;
                 }
 
@@ -588,16 +677,37 @@ namespace LinuxSampler {
                     printf("-> STMT_LIST subidx=%d\n", frame.subindex);
                     #endif
                     Statements* stmts = (Statements*) frame.statement;
-                    if (stmts->statement(frame.subindex)) {
-                        ctx->pushStack(
-                            stmts->statement(frame.subindex++)
-                        );
+                    if (!frame.state) {
+                        Statement* dependencyStmt =
+                            stmts->dependencyStatement(frame.subindex);
+                        if (!dependencyStmt) {
+                            frame.subindex = 0;
+                            frame.state = 1;
+                            goto list_state_1;
+                        }
+                        ctx->pushOpStack(dependencyStmt);
+                        ++frame.subindex;
                     } else {
-                        #if DEBUG_SCRIPTVM_CORE
-                        _printIndents(frameIdx);
-                        printf("[END OF LIST] subidx=%d\n", frame.subindex);
-                        #endif
-                        ctx->popStack();
+                        list_state_1:
+                        if (!frame.subindex) {
+                            if (stmts->needsNewDataStackFrame())
+                                ctx->pushDataStack(stmts);
+                            stmts->constructLocalData();
+                            frame.subindex = stmts->startStatement();
+                        }
+                        if (Statement* nextStmt = stmts->statement(frame.subindex)) {
+                            ctx->pushOpStack(nextStmt);
+                            ++frame.subindex;
+                        } else {
+                            #if DEBUG_SCRIPTVM_CORE
+                            _printIndents(frameIdx);
+                            printf("[END OF LIST] subidx=%d\n", frame.subindex);
+                            #endif
+                            stmts->destructLocalData();
+                            if (stmts->needsNewDataStackFrame())
+                                ctx->popDataStack();
+                            ctx->popOpStack();
+                        }
                     }
                     break;
                 }
@@ -607,18 +717,31 @@ namespace LinuxSampler {
                     _printIndents(frameIdx);
                     printf("-> STMT_BRANCH\n");
                     #endif
-                    if (frame.subindex < 0) ctx->popStack();
+                    if (frame.subindex < 0) ctx->popOpStack();
                     else {
                         BranchStatement* branchStmt = (BranchStatement*) frame.statement;
-                        frame.subindex =
-                            (decltype(frame.subindex))
-                                branchStmt->evalBranch();
-                        if (frame.subindex >= 0) {
-                            ctx->pushStack(
-                                branchStmt->branch(frame.subindex)
-                            );
-                            frame.subindex = -1;
-                        } else ctx->popStack();
+                        if (!frame.state) {
+                            Statement* dependencyStmt =
+                                branchStmt->dependencyStatement(frame.subindex);
+                            if (!dependencyStmt) {
+                                frame.subindex = 0;
+                                frame.state = 1;
+                                goto branch_state_1;
+                            }
+                            ctx->pushOpStack(dependencyStmt);
+                            ++frame.subindex;
+                        } else {
+                            branch_state_1:
+                            frame.subindex =
+                                (decltype(frame.subindex))
+                                    branchStmt->evalBranch();
+                            if (frame.subindex >= 0) {
+                                ctx->pushOpStack(
+                                    branchStmt->branch(frame.subindex)
+                                );
+                                frame.subindex = -1;
+                            } else ctx->popOpStack();
+                        }
                     }
                     break;
                 }
@@ -629,17 +752,31 @@ namespace LinuxSampler {
                     printf("-> STMT_LOOP\n");
                     #endif
                     While* whileStmt = (While*) frame.statement;
-                    if (whileStmt->evalLoopStartCondition() && whileStmt->statements()) {
-                        ctx->pushStack(
-                            whileStmt->statements()
-                        );
-                        if (flags == STMT_SUCCESS && !synced &&
-                            instructionsCounter > SCRIPTVM_MAX_INSTR_PER_CYCLE_SOFT)
-                        {
-                            flags = StmtFlags_t(STMT_SUSPEND_SIGNALLED);
-                            ctx->suspendMicroseconds = SCRIPT_VM_FORCE_SUSPENSION_MICROSECONDS;
+                    if (!frame.state) {
+                        Statement* dependencyStmt =
+                            whileStmt->dependencyStatement(frame.subindex);
+                        if (!dependencyStmt) {
+                            frame.subindex = 0;
+                            frame.state = 1;
+                            goto while_state_1;
                         }
-                    } else ctx->popStack();
+                        ctx->pushOpStack(dependencyStmt);
+                        ++frame.subindex;
+                    } else {
+                        while_state_1:
+                        Statements* whileStmts;
+                        if (whileStmt->evalLoopStartCondition() && (whileStmts = whileStmt->statements())) {
+                            ctx->pushOpStack(whileStmts);
+                            if (flags == STMT_SUCCESS && !synced &&
+                                instructionsCounter > SCRIPTVM_MAX_INSTR_PER_CYCLE_SOFT)
+                            {
+                                flags = StmtFlags_t(STMT_SUSPEND_SIGNALLED);
+                                ctx->suspendMicroseconds = SCRIPT_VM_FORCE_SUSPENSION_MICROSECONDS;
+                            }
+                            frame.subindex = 0;
+                            frame.state = 0;
+                        } else ctx->popOpStack();
+                    }
                     break;
                 }
 
@@ -649,31 +786,42 @@ namespace LinuxSampler {
                     printf("-> STMT_SYNC\n");
                     #endif
                     SyncBlock* syncStmt = (SyncBlock*) frame.statement;
-                    if (!frame.subindex++ && syncStmt->statements()) {
+                    Statements* syncStmts;
+                    if (!frame.subindex++ && (syncStmts = syncStmt->statements())) {
                         ++synced;
-                        ctx->pushStack(
-                            syncStmt->statements()
-                        );
+                        ctx->pushOpStack(syncStmts);
                     } else {
-                        ctx->popStack();
+                        ctx->popOpStack();
                         --synced;
                     }
                     break;
                 }
 
                 case STMT_NOOP:
-                    break; // no operation like the name suggests
+                    #if DEBUG_SCRIPTVM_CORE
+                    _printIndents(frameIdx);
+                    printf("-> STMT_NOOP\n");
+                    #endif
+                    ctx->popOpStack();
+                    break;
             }
 
             if (flags & STMT_RETURN_SIGNALLED) {
                 flags = StmtFlags_t(flags & ~STMT_RETURN_SIGNALLED);
-                for (; frameIdx >= 0; ctx->popStack()) {
-                    frame = ctx->stack[frameIdx];
-                    if (frame.statement->statementType() == STMT_SYNC) {
+                for (; frameIdx >= 0; ctx->popOpStack()) {
+                    frame = ctx->opStack[frameIdx];
+                    StmtType_t stmtType = frame.statement->statementType();
+                    if (stmtType == STMT_SYNC) {
                         --synced;
-                    } else if (dynamic_cast<Subroutine*>(frame.statement)) {
-                        ctx->popStack();
-                        break; // stop here
+                    } else if (stmtType == STMT_LIST) {
+                        Statements* stmts = (Statements*) frame.statement;
+                        stmts->destructLocalData();
+                        if (stmts->needsNewDataStackFrame())
+                            ctx->popDataStack();
+                        if (dynamic_cast<UserFunctionCall*>(stmts)) {
+                            ctx->popOpStack();
+                            break; // stop here
+                        }
                     }
                 }
             }
